@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -109,6 +109,9 @@ foreach ($iconList as $icon) {
 	$data['iconList'][$icon['imageid']] = $icon['name'];
 }
 
+reset($iconList);
+$data['default_imageid'] = key($iconList);
+
 $inventoryFields = getHostInventories();
 foreach ($inventoryFields as $field) {
 	$data['inventoryList'][$field['nr']] = $field['title'];
