@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -231,7 +231,7 @@ class CHttpTestManager {
 
 		// Old items must be deleted prior to createStepsReal() since identical items cannot be created in DB.
 		if ($deleteStepItemIds) {
-			API::Item()->delete($deleteStepItemIds, true);
+			CItemManager::delete($deleteStepItemIds);
 		}
 
 		foreach ($httptests as $key => $httptest) {
@@ -274,31 +274,6 @@ class CHttpTestManager {
 			}
 		}
 
-		/*
-		 * Unset unchanged properties, otherwise, host web scenarios will inherit them and their custom applications
-		 * will be overwritten.
-		 */
-		$properties_to_compare = ['applicationid', 'ssl_cert_file', 'ssl_key_password', 'ssl_key_file', 'verify_peer',
-			'verify_host'
-		];
-
-		foreach ($httptests as &$http_test) {
-			foreach ($properties_to_compare as $property) {
-				// Do not compare unexisting properties.
-				if (!array_key_exists($property, $http_test)) {
-					continue;
-				}
-
-				// Unset unchanged properties.
-				if (($property === 'applicationid'
-						&& bccomp($http_test[$property], $db_httptests[$http_test['httptestid']][$property]) == 0)
-						|| $http_test[$property] === $db_httptests[$http_test['httptestid']][$property]) {
-					unset($http_test[$property]);
-				}
-			}
-		}
-		unset($http_test);
-
 		$this->updateHttpTestFields($httptests, 'update');
 
 		return $httptests;
@@ -421,7 +396,9 @@ class CHttpTestManager {
 				elseif (isset($hostHttpTest['byName'][$httpTest['name']])) {
 					$exHttpTest = $hostHttpTest['byName'][$httpTest['name']];
 
-					if (bccomp($exHttpTest['templateid'], $httpTestId) == 0 || !$this->compareHttpSteps($httpTest, $exHttpTest)) {
+					if (bccomp($exHttpTest['templateid'], $httpTestId) == 0
+							|| $exHttpTest['templateid'] != 0
+							|| !$this->compareHttpSteps($httpTest, $exHttpTest)) {
 						$host = DBfetch(DBselect('SELECT h.name FROM hosts h WHERE h.hostid='.zbx_dbstr($hostId)));
 						throw new Exception(
 							_s('Web scenario "%1$s" already exists on host "%2$s".', $exHttpTest['name'], $host['name'])
