@@ -437,17 +437,23 @@ var hintBox = {
 	 */
 	bindEvents: function () {
 		jQuery(document).on('keydown click mouseenter mouseleave', '[data-hintbox=1]', function (e) {
-			var target = jQuery(this);
+
+			if (jQuery(this).hasClass('hint-item')) {
+				var target = jQuery(this).siblings('.main-hint');
+			}
+			else {
+				var target = jQuery(this);
+			}
 
 			switch (e.type) {
 				case 'mouseenter':
-					hintBox.showHint(e, this, target.next('.hint-box').html(), target.data('hintbox-class'), false,
+					hintBox.showHint(e, target[0], target.next('.hint-box').html(), target.data('hintbox-class'), false,
 						target.data('hintbox-style')
 					);
 					break;
 
 				case 'mouseleave':
-					hintBox.hideHint(this, false);
+					hintBox.hideHint(target[0], false);
 					break;
 
 				case 'keydown':
@@ -459,7 +465,7 @@ var hintBox = {
 						e.clientY = offset.top - w.scrollTop() + (target.height() / 2);
 						e.preventDefault();
 
-						hintBox.showStaticHint(e, this, target.data('hintbox-class'), false,
+						hintBox.showStaticHint(e, target[0], target.data('hintbox-class'), false,
 							target.data('hintbox-style')
 						);
 					}
@@ -467,7 +473,7 @@ var hintBox = {
 
 				case 'click':
 					if (target.data('hintbox-static') == 1) {
-						hintBox.showStaticHint(e, this, target.data('hintbox-class'), false,
+						hintBox.showStaticHint(e, target[0], target.data('hintbox-class'), false,
 							target.data('hintbox-style')
 						);
 					}
@@ -546,6 +552,7 @@ var hintBox = {
 
 			target.isStatic = true;
 			hintBox.showHint(e, target, hintText, className, true, styles);
+			jQuery(target).data('return-control', jQuery(e.target));
 
 			if (resizeAfterLoad) {
 				hintText.one('load', function(e) {
@@ -660,6 +667,9 @@ var hintBox = {
 			delete target.hintBoxItem;
 
 			if (target.isStatic) {
+				if (jQuery(target).data('return-control') !== 'undefined') {
+					jQuery(target).data('return-control').focus();
+				}
 				delete target.isStatic;
 			}
 		}
